/** 
 *  Yudit Unicode Editor Source File
 *
 *  GNU Copyright (C) 2002  Gaspar Sinai <gsinai@yudit.org>  
 *  GNU Copyright (C) 2001  Gaspar Sinai <gsinai@yudit.org>  
 *  GNU Copyright (C) 2000  Gaspar Sinai <gsinai@yudit.org>  
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License, version 2,
 *  dated June 1991. See file COPYYING for details.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <stdio.h>
#include <swindow/SAwt.h>
#include <swindow/SFontTTF.h>
#include "stoolkit/SEvent.h"
#include "stoolkit/SMatrix.h"
#include "stoolkit/SStringVector.h"
#include "stoolkit/SUniMap.h"
#include <stdlib.h>

/* THIS WILL BE DRAWN */
unsigned int drawThis = 0x0041;

#define FONT_SIZE 100
/* fg bg */
SPen pen (SColor(0.0,1.0,0.0, 1.0), SColor(0.0, 0.0, 0.0, 1.0), 1.0);

class Listener : public SWindowListener
{
public:
  SFontTTF* ttf;
  SS_Matrix2D matrix;
  Listener(SFontTTF* ttfin) : SWindowListener()
  {
     ttf=ttfin; 
     double scale = ttf->scale() * FONT_SIZE;
     matrix.scale (scale, -scale);
     matrix.translate (50, FONT_SIZE+50);
  }
  ~Listener() {}
  virtual void redraw(SWindow* w, int x, int y,
    unsigned int width ,unsigned int height)
  {
    SS_UCS4 u4 = drawThis;
    SString key = SString((char*)&u4, sizeof (u4));
    double wi = 0.0;
#if 0
    for (unsigned int i=0; i<100000; i++)
    {
      ttf->width (matrix, u4, &wi);
    }
#endif
    bool hasw =  ttf->width (matrix, u4, &wi);
    fprintf (stderr, "HasWidth=%d width=%g\n", (int)hasw, wi);
    if (!hasw) return;
    for (unsigned int i=0; i<1; i++)
    {
      if (!w->newpath (matrix.t0, matrix.t1, key))
      {
        fprintf (stderr, "Draw..\n");
        ttf->draw (w, matrix, u4);
      }
      w->fill (pen);
    }
  }
  virtual bool windowClose (SWindow* w)
  {
    fprintf (stderr, "Window closing..\n");
    exit (0);
    return true;
  }
};


int
main (int argc, char* argv[])
{
  //SUniMap::guessPath ();
  SAwt awt(true);
  
  if (!awt.implemented())
  {
    fprintf (stderr, "AWT is not implemented!\n");
    return 0;
  }
  if (argc == 1) 
  {
    fprintf (stderr, "usgae: rich /dir/file.ttf [U+XXXX]\n");
    exit (0);
  }
  SStringVector v(argv[1], ":");
  SFile file (v[0]);
  if (file.size()<=0)
  {
    fprintf (stderr, "Can not open fontfile %s\n", argv[1]);
    exit (0);
  }
  SString encoding = (v.size()>1) ? SString(v[1]) : SString();
  SFontTTF* ttf = new SFontTTF (file, encoding);
  if (!ttf->isOK())
  {
    fprintf (stderr, "Can not use fontfile %s\n", argv[1]);
    exit (0);
  }
  if (argc > 2)
  {
    sscanf (argv[2], "U+%X", &drawThis);
  }
  SWindow* w = awt.getWindow(new Listener(ttf), "Rich's Test");
  w->setBackground (pen.getBackground());
    w->fill (SColor(0.0,1.0,0.0, 1.0));
  w->resize (200, 200);
  w->show();
  SEventHandler::start();
  return (0);
}
