/** 
 *  Odor Analytics Source File
 *
 *  Copyright (C) 2002  Gaspar Sinai <gsinai@yudit.org>  
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "TMatrix.h"

static void usage ();

char* tmops[] = { 
  "scan", "print", "pm",   "pe",   "ps", 
  "se",   "tr",    "unit", "zero", "rand100",
  "eij",  "sort",  "de",   "pow",  "log", 
  "abs",  "add",   "exp", "me",
};

typedef enum { 
  TM_SCAN=1, TM_PRINT, TM_PM,   TM_PE,   TM_PS,
  TM_SE,     TM_TR,    TM_UNIT, TM_ZERO, TM_RAND100,
  TM_EIJ,    TM_SORT,  TM_DE,   TM_POW,   TM_LOG,
  TM_ABS,    TM_ADD,   TM_EXP,  TM_ME
} TM_Operator;

char tmp[256];

/**
 *  This module writes a matrix from ordered floating point data.
 */
int
main (int argc, char* argv[])
{
  SString outs;
  SStringVector ins;
  unsigned int rows = 0;
  unsigned int cols = 0;
  unsigned int i;
  double scalar=0.0;
  unsigned int cellrows = 0;
  unsigned int cellcols = 0;
  int introw=-1;
  int intcol=-1;
  SString indexout;

  FILE* in = stdin;

  if (argc < 2)
  {
    usage ();
    return (1);
  }
  TM_Operator op = (TM_Operator)0;
  for (i=0; i<sizeof (tmops)/sizeof (char*); i++)
  {
    if (strcmp (argv[1], tmops[i])==0)
    {
      op = (TM_Operator) (i+1);
      break;
    }
  }
  if (op==(TM_Operator)0)
  {
    fprintf (stderr, "Operand not found: %s\n", argv[1]);
    usage ();
    return (1);
  }
  for (i=2; i<(unsigned int)argc; i++)
  {
    if (sscanf (argv[i], "out=%255s", tmp) == 1)
    {
      tmp[255] = 0;
      outs = SString(tmp);
    }
    else if (sscanf (argv[i], "iout=%255s", tmp) == 1)
    {
      tmp[255] = 0;
      indexout = SString(tmp);
    }
    else if (sscanf (argv[i], "size=%255s", tmp) == 1)
    {
      tmp[255] = 0;
      int r, c;
      if (!(sscanf (tmp, "%dx%d", &r, &c) || sscanf (tmp, "%dX%d", &r, &c)
         || r <=0 || c<=0))
      {
        fprintf (stderr, "tm: bad size %s.\n", tmp);
        return 1;
      }
      rows = (unsigned int)r;
      cols = (unsigned int)c;
    }
    else if (sscanf (argv[i], "cell=%255s", tmp) == 1)
    {
      tmp[255] = 0;
      int r, c;
      if (!(sscanf (tmp, "%d,%d", &r, &c) || sscanf (tmp, "%d,%d", &r, &c)
         || r <=0 || c<=0))
      {
        fprintf (stderr, "tm: bad cell parameter %s.\n", tmp);
        return 1;
      }
      cellrows = (unsigned int)r;
      cellcols = (unsigned int)c;
    }
    else if (sscanf (argv[i], "scalar=%lf", &scalar) == 1)
    {
    }
    else if (sscanf (argv[i], "k=%lf", &scalar) == 1)
    {
    }
    else if (sscanf (argv[i], "row=%d", &introw) == 1)
    {
      if (introw < 0)
      {
        fprintf (stderr, "tm: bad row index: %d\n", introw);
        return 1;
      }
    }
    else if (sscanf (argv[i], "col=%d", &intcol) == 1)
    {
      if (intcol < 0)
      {
        fprintf (stderr, "tm: bad col index: %d\n", intcol);
        return 1;
      }
    }
    else if (op == TM_SCAN  && i+1==(unsigned int)argc)
    { 
      in = fopen (argv[i], "r"); 
      if (in==NULL)
      {
        fprintf (stderr, "tm: can read file %s.\n", argv[i]);
        return 1;
      }
    } 
    else if (op == TM_PRINT  && i+1==(unsigned int)argc)
    { 
      ins.append (argv[i]);
    } 
    else 
    {
      switch (op)
      {
      /* These need one matrix argument */
      case TM_PM:
      case TM_PE:
      case TM_SE:
      case TM_DE:
      case TM_ME:
        if (i+2 != (unsigned int)argc)
        {
          fprintf (stderr, "tm: Too few or too many input files (%d).\n",
               argc-(int)i);
          usage ();
          return 1;
        }
        ins.append (argv[i]);
        i++;
        ins.append (argv[i]);
        break;
      /* These need no matrix arguments */
      case TM_TR:
      case TM_PS:
      case TM_ABS:
      case TM_EXP:
      case TM_POW:
      case TM_LOG:
      case TM_ADD:
      case TM_SORT:
        if (i+1 != (unsigned int)argc)
        {
          fprintf (stderr, "tm: Too few or too many input files (%d).\n",
               argc-(int)i);
          usage ();
          return 1;
        }
        ins.append (argv[i]);
        break;
      default:
         usage ();
         return 1;
      }
    }
  }
  if (op != TM_PRINT && outs.size()==0)
  {
    fprintf (stderr, "tm: not enough arguments.\n");
    usage ();
    return 1;
  }
  
  switch (op)
  {
  case TM_SCAN:
    {
      if (in == stdin)
      {
        fprintf (stderr, "tm scan: processing data from stdin. Expectin %d foating point numbers:\n", rows*cols);
      }
      SFile file(outs);
      TMatrix matrix (file, (unsigned int) rows, (unsigned int) cols);
      if (!matrix.isOK() || !matrix.scan (in))
      {
        SString err = matrix.getError();
        fprintf (stderr, "tm scan: %*.*s.\n", SSARGS(err));
        return 1;
      }
      fclose (in);
    }
    break;
  case TM_PRINT:
    {
      if (ins.size()!=1)
      {
        usage();
        return 1;
      }
      SString fname = outs;
      fname.append ((char)0);
      FILE* out = (outs.size()==0) ? stdout : fopen (fname.array(), "w"); 

      if (out==NULL)
      {
        fprintf (stderr, "tm print: can open file for writing %s.\n", 
           fname.array());
        return 1;
      }
      SFile file (ins[0]);
      TMatrix matrix (file);

      if (!matrix.isOK())
      {
        SString err = matrix.getError();
        fprintf (stderr, "tm print: %*.*s.\n", SSARGS(err));
        return 1;
      }
      if (!matrix.print (out))
      {
        SString err = matrix.getError();
        fprintf (stderr, "tm print: %*.*s.\n", SSARGS(err));
        return 1;
      }
      fclose (out);
    }
    break;
  case TM_PM:
    {
      if (ins.size()!=2)
      {
        usage();
        return 1;
      }
      SFile file0 = ins[0];
      TMatrix matrix0 (file0);
      if (!matrix0.isOK())
      {
        SString err = matrix0.getError();
        fprintf (stderr, "tm pm: %*.*s.\n", SSARGS(err));
        return 1;
      }

      SFile file1 = ins[1];
      TMatrix matrix1 (file1);
      if (!matrix1.isOK())
      {
        SString err = matrix1.getError();
        fprintf (stderr, "tm pm: %*.*s.\n", SSARGS(err));
        return 1;
      }
      SFile file(outs);
      TMatrix result = matrix0.pm (file, matrix1);
      if (!result.isOK())
      {
        SString err = result.getError();
        fprintf (stderr, "tm pm: %*.*s.\n", SSARGS(err));
        return 1;
      }
    }
    break;
  case TM_PE:
    {
      if (ins.size()!=2)
      {
        usage();
        return 1;
      }
      SFile file0 = ins[0];
      TMatrix matrix0 (file0);
      if (!matrix0.isOK())
      {
        SString err = matrix0.getError();
        fprintf (stderr, "tm pe: %*.*s.\n", SSARGS(err));
        return 1;
      }

      SFile file1 = ins[1];
      TMatrix matrix1 (file1);
      if (!matrix1.isOK())
      {
        SString err = matrix1.getError();
        fprintf (stderr, "tm pe: %*.*s.\n", SSARGS(err));
        return 1;
      }
      SFile file(outs);
      TMatrix result = matrix0.pe (file, matrix1);
      if (!result.isOK())
      {
        SString err = result.getError();
        fprintf (stderr, "tm pe: %*.*s.\n", SSARGS(err));
        return 1;
      }
    }
    break;
  case TM_DE:
    {
      if (ins.size()!=2)
      {
        usage();
        return 1;
      }
      SFile file0 = ins[0];
      TMatrix matrix0 (file0);
      if (!matrix0.isOK())
      {
        SString err = matrix0.getError();
        fprintf (stderr, "tm de: %*.*s.\n", SSARGS(err));
        return 1;
      }

      SFile file1 = ins[1];
      TMatrix matrix1 (file1);
      if (!matrix1.isOK())
      {
        SString err = matrix1.getError();
        fprintf (stderr, "tm de: %*.*s.\n", SSARGS(err));
        return 1;
      }
      SFile file(outs);
      TMatrix result = matrix0.de (file, matrix1);
      if (!result.isOK())
      {
        SString err = result.getError();
        fprintf (stderr, "tm de: %*.*s.\n", SSARGS(err));
        return 1;
      }
    }
    break;
  case TM_ME:
    {
      if (ins.size()!=2)
      {
        usage();
        return 1;
      }
      SFile file0 = ins[0];
      TMatrix matrix0 (file0);
      if (!matrix0.isOK())
      {
        SString err = matrix0.getError();
        fprintf (stderr, "tm me: %*.*s.\n", SSARGS(err));
        return 1;
      }

      SFile file1 = ins[1];
      TMatrix matrix1 (file1);
      if (!matrix1.isOK())
      {
        SString err = matrix1.getError();
        fprintf (stderr, "tm me: %*.*s.\n", SSARGS(err));
        return 1;
      }
      SFile file(outs);
      TMatrix result = matrix0.me (file, matrix1);
      if (!result.isOK())
      {
        SString err = result.getError();
        fprintf (stderr, "tm me: %*.*s.\n", SSARGS(err));
        return 1;
      }
    }
    break;
  case TM_SE:
    {
      if (ins.size()!=2)
      {
        usage();
        return 1;
      }
      SFile file0 = ins[0];
      TMatrix matrix0 (file0);
      if (!matrix0.isOK())
      {
        SString err = matrix0.getError();
        fprintf (stderr, "tm se: %*.*s.\n", SSARGS(err));
        return 1;
      }

      SFile file1 = ins[1];
      TMatrix matrix1 (file1);
      if (!matrix1.isOK())
      {
        SString err = matrix1.getError();
        fprintf (stderr, "tm se: %*.*s.\n", SSARGS(err));
        return 1;
      }
      SFile file(outs);
      TMatrix result = matrix0.se (file, matrix1);
      if (!result.isOK())
      {
        SString err = result.getError();
        fprintf (stderr, "tm se: %*.*s.\n", SSARGS(err));
        return 1;
      }
    }
    break;
  case TM_PS:
    {
      if (ins.size()!=1)
      {
        usage();
        return 1;
      }
      SFile file0 = ins[0];
      TMatrix matrix0 (file0);
      if (!matrix0.isOK())
      {
        SString err = matrix0.getError();
        fprintf (stderr, "tm ps: %*.*s.\n", SSARGS(err));
        return 1;
      }

      SFile file(outs);
      TMatrix result = matrix0.ps (file, scalar);
      if (!result.isOK())
      {
        SString err = result.getError();
        fprintf (stderr, "tm ps: %*.*s.\n", SSARGS(err));
        return 1;
      }
    }
    break;
  case TM_POW:
    {
      if (ins.size()!=1)
      {
        usage();
        return 1;
      }
      SFile file0 = ins[0];
      TMatrix matrix0 (file0);
      if (!matrix0.isOK())
      {
        SString err = matrix0.getError();
        fprintf (stderr, "tm pow: %*.*s.\n", SSARGS(err));
        return 1;
      }

      SFile file(outs);
      TMatrix result = matrix0.pow (file, scalar);
      if (!result.isOK())
      {
        SString err = result.getError();
        fprintf (stderr, "tm pow: %*.*s.\n", SSARGS(err));
        return 1;
      }
    }
    break;
  case TM_ADD:
    {
      if (ins.size()!=1)
      {
        usage();
        return 1;
      }
      SFile file0 = ins[0];
      TMatrix matrix0 (file0);
      if (!matrix0.isOK())
      {
        SString err = matrix0.getError();
        fprintf (stderr, "tm add: %*.*s.\n", SSARGS(err));
        return 1;
      }

      SFile file(outs);
      TMatrix result = matrix0.add (file, scalar);
      if (!result.isOK())
      {
        SString err = result.getError();
        fprintf (stderr, "tm add: %*.*s.\n", SSARGS(err));
        return 1;
      }
    }
    break;
  case TM_TR:
    {
      if (ins.size()!=1)
      {
        fprintf (stderr, "tm: tr needs exatcly 1 input file.\n");
        usage();
        return 1;
      }
      SFile file0 = ins[0];
      TMatrix matrix0 (file0);
      if (!matrix0.isOK())
      {
        SString err = matrix0.getError();
        fprintf (stderr, "tm tr: %*.*s.\n", SSARGS(err));
        return 1;
      }

      SFile file(outs);
      TMatrix result = matrix0.tr (file);
      if (!result.isOK())
      {
        SString err = result.getError();
        fprintf (stderr, "tm tr: %*.*s.\n", SSARGS(err));
        return 1;
      }
    }
    break;
  case TM_ABS:
    {
      if (ins.size()!=1)
      {
        fprintf (stderr, "tm: abs needs exatcly 1 input file.\n");
        usage();
        return 1;
      }
      SFile file0 = ins[0];
      TMatrix matrix0 (file0);
      if (!matrix0.isOK())
      {
        SString err = matrix0.getError();
        fprintf (stderr, "tm abs: %*.*s.\n", SSARGS(err));
        return 1;
      }

      SFile file(outs);
      TMatrix result = matrix0.abs (file);
      if (!result.isOK())
      {
        SString err = result.getError();
        fprintf (stderr, "tm abs: %*.*s.\n", SSARGS(err));
        return 1;
      }
    }
    break;
  case TM_EXP:
    {
      if (ins.size()!=1)
      {
        fprintf (stderr, "tm: exp needs exatcly 1 input file.\n");
        usage();
        return 1;
      }
      SFile file0 = ins[0];
      TMatrix matrix0 (file0);
      if (!matrix0.isOK())
      {
        SString err = matrix0.getError();
        fprintf (stderr, "tm exp: %*.*s.\n", SSARGS(err));
        return 1;
      }

      SFile file(outs);
      TMatrix result = matrix0.exp (file);
      if (!result.isOK())
      {
        SString err = result.getError();
        fprintf (stderr, "tm exp: %*.*s.\n", SSARGS(err));
        return 1;
      }
    }
    break;
  case TM_LOG:
    {
      if (ins.size()!=1)
      {
        fprintf (stderr, "tm: log needs exatcly 1 input file.\n");
        usage();
        return 1;
      }
      SFile file0 = ins[0];
      TMatrix matrix0 (file0);
      if (!matrix0.isOK())
      {
        SString err = matrix0.getError();
        fprintf (stderr, "tm log: %*.*s.\n", SSARGS(err));
        return 1;
      }

      SFile file(outs);
      TMatrix result = matrix0.log (file);
      if (!result.isOK())
      {
        SString err = result.getError();
        fprintf (stderr, "tm log: %*.*s.\n", SSARGS(err));
        return 1;
      }
    }
    break;
  case TM_ZERO:
    {
      if (ins.size()!=0)
      {
        usage();
        return 1;
      }
      SFile file(outs);
      TMatrix result = TMatrix::zero (file, rows, cols);
      if (!result.isOK())
      {
        SString err = result.getError();
        fprintf (stderr, "tm zero: %*.*s.\n", SSARGS(err));
        return 1;
      }
    }
    break;
  case TM_UNIT:
    {
      if (cols != rows)
      {
        fprintf (stderr, "tm unit: rows != cols\n");
        return 1;
      }
      if (ins.size()!=0)
      {
        usage();
        return 1;
      }
      SFile file(outs);
      TMatrix result = TMatrix::unit (file, rows);
      if (!result.isOK())
      {
        SString err = result.getError();
        fprintf (stderr, "tm unit: %*.*s.\n", SSARGS(err));
        return 1;
      }
    }
    break;
  case TM_RAND100:
    {
      if (ins.size()!=0)
      {
        usage();
        return 1;
      }
      SFile file(outs);
      TMatrix result = TMatrix::rand100 (file, rows, cols);
      if (!result.isOK())
      {
        SString err = result.getError();
        fprintf (stderr, "tm rand100: %*.*s.\n", SSARGS(err));
        return 1;
      }
    }
    break;
  case TM_EIJ:
    {
      if (cellrows >= rows || cellcols >= cols)
      {
        fprintf (stderr, "tm eij: bad index (cell).\n");
        return 1;
      }
      if (ins.size()!=0)
      {
        usage();
        return 1;
      }
      SFile file(outs);
      TMatrix result = TMatrix::zero (file, rows, cols);
      if (!result.isOK())
      {
        SString err = result.getError();
        fprintf (stderr, "tm eij: %*.*s.\n", SSARGS(err));
        return 1;
      }
      result.set (cellrows, cellcols, (FType)1);
    }
    break;
  case TM_SORT:
    {
      if (ins.size()!=1)
      {
        fprintf (stderr, "tm: tr needs exatcly 1 input file.\n");
        usage();
        return 1;
      }
      if (intcol >= 0 && introw >= 0)
      {
        usage();
        return 1;
      }
      if (intcol <= 0 && introw <= 0)
      {
        usage();
        return 1;
      }
      SFile file0 = ins[0];
      TMatrix matrix0 (file0);
      if (!matrix0.isOK())
      {
        SString err = matrix0.getError();
        fprintf (stderr, "tm sort: %*.*s.\n", SSARGS(err));
        return 1;
      }

      SFile file(outs);
      SFile ifile(indexout);
      TMatrix result = matrix0.sort (file, ifile,
             (introw>=0) ? (unsigned int) introw : (unsigned int) intcol,
             (introw>=0));
      if (!result.isOK())
      {
        SString err = result.getError();
        fprintf (stderr, "tm tp: %*.*s.\n", SSARGS(err));
        return 1;
      }
    }
    break;
  default:
    break;
    usage();
  }
  return 0;
}

static void
usage ()
{
  fprintf (stderr, "tm: Tibor Matrix (tm)\n");
  fprintf (stderr, "usage:   \n");
  fprintf (stderr, "convert into native: tm scan size=rowsxcols out=output-matrix [input-file]\n");
  fprintf (stderr, "convert from native: tm print input-matrix\n");
  fprintf (stderr, "matrix product     : tm pm out=output-matrix input-matrix-0 input-matrix-1\n");
  fprintf (stderr, "element product    : tm pe  out=output-matrix input-matrix-0 input-matrix-1\n");
  fprintf (stderr, "element division   : tm de  out=output-matrix input-matrix-0 input-matrix-1\n");
  fprintf (stderr, "element sum        : tm se  out=output-matrix input-matrix-0 input-matrix-1\n");
  fprintf (stderr, "element sub        : tm me  out=output-matrix input-matrix-0 input-matrix-1\n");
  fprintf (stderr, "sort               : tm sort {col=N|row=N} out=output-matrix [iout=index-out] input-matrix\n");
  fprintf (stderr, "index              : tm sort {col=N|row=N} out=output-matrix index=index-input-matrix-key\n");
  fprintf (stderr, "transponate        : tm tr out=output-matrix input-matrix\n");
  fprintf (stderr, "unit matrix        : tm unit size=NxN out=output-matrix\n");
  fprintf (stderr, "zero matrix        : tm zero size=NxN out=output-matrix\n");
  fprintf (stderr, "random matrix      : tm rand100 size=NxN out=output-matrix\n");
  fprintf (stderr, "power              : tm pow k=n out=output-matrix input-matrix\n");
  fprintf (stderr, "add sclar          : tm add k=n out=output-matrix input-matrix\n");
  fprintf (stderr, "scalar product     : tm ps k=N out=output-matrix input-matrix-0\n");
  fprintf (stderr, "absolute value     : tm abs out=output-matrix input-matrix\n");
  fprintf (stderr, "natural logarithm  : tm log k=n out=output-matrix input-matrix\n");
  fprintf (stderr, "exponential value  : tm exp k=n out=output-matrix input-matrix\n");
  fprintf (stderr, "eij matrix         : tm eij  size=NxM cell=n,m out=output-matrix\n");
  fprintf (stderr, "example: tm scan size=2x2 out=matrix.mat inputfile.txt\n");
  fprintf (stderr, "         tm print matrix.mat\n");
  fprintf (stderr, "         tm pm out=product.mat matrix0.mat matrix1.mat\n");
}
